﻿<#
.SYNOPSIS
	Run as a scheduled task, grants Lync/Skype for Business policy to members of an Active Directory security group

.DESCRIPTION
	Run as a scheduled task, grants Lync/Skype for Business policy to members of an Active Directory security group. The name of the group, and the type and name of the policy to be granted are all customizable.

.NOTES
  Version    	      		: v1.7 - See changelog at http://www.ucunleashed.com/1346
  Wish list							:
  Rights Required				: local admin on server it's running on; CsUserAdministrator or CsAdministrator in Lync/Skype for Business Server
  Sched Task Required		: Yes - install mode will automatically create scheduled task to run every 4 hours
  Lync/Skype4B Version  : 2010 and 2013, and Skype for Business Server 2015
	Author/Copyright			: © Pat Richard, Skype for Business MVP - All Rights Reserved
  Email/Blog/Twitter		: pat@innervation.com 	http://www.ucunleashed.com @patrichard
  Donations             : https://www.paypal.me/PatRichard
  Dedicated Post				: http://www.ucunleashed.com/1348
  Disclaimer            : You running this script means you won't blame author(s) if this breaks your stuff. This script is
                          provided AS IS without warranty of any kind. Author(s) disclaim all implied warranties including,
                          without limitation, any implied warranties of merchantability or of fitness for a particular
                          purpose. The entire risk arising out of the use or performance of the sample scripts and
                          documentation remains with you. In no event shall author(s) be liable for any damages whatsoever
                          (including, without limitation, damages for loss of business profits, business interruption,
                          loss of business information, or other pecuniary loss) arising out of the use of or inability
                          to use the script or documentation. Neither this script, nor any part of it other than those
                          parts that are explicitly copied from others, may be republished without author(s) express written 
                          permission.
  Acknowledgements 			: http://jeffwouters.nl/index.php/2012/06/powershell-function-to-get-all-nested-group-members-in-active-directory/
	Assumptions						: ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
  Limitations						:
  Known issues					: None yet, but I'm sure you'll find some!

.LINK
	http://www.ucunleashed.com/1348

.EXAMPLE
	.\Grant-CsPolicyByADGroup.ps1

	Description
	-----------
	Looks at members of a defined AD group, and grants a Lync policy to each

.EXAMPLE
	.\Grant-CsPolicyByADGroup.ps1 -install

	Description
	-----------
	Creates the scheduled task for the script to run everyday at 6am. It will prompt for the password for the currently logged on user.

.INPUTS
	None. You cannot pipe objects to this script
#>
#Requires -Version 3.0

[CmdletBinding(SupportsShouldProcess, DefaultParameterSetName = "Default")]
param(
	# DN of group to check for membership
	[Parameter(ValueFromPipelineByPropertyName, ParameterSetName = "Default")]
	[ValidateNotNullOrEmpty()]
	[string] $GroupDN,

	# Name of policy to assign to users
	[Parameter(ValueFromPipelineByPropertyName, ParameterSetName = "Default")]
	[ValidateNotNullOrEmpty()]
	[string] $PolicyName,

	# Type of policy to assign to users
	[parameter(ValueFromPipelineByPropertyName, ParameterSetName = "Default")]
	[ValidateNotNullOrEmpty()]
	[ValidateSet("Archiving","Client","ClientVersion","Conferencing","ExternalAccess","HostedVoicemail","Location","Mobility","Pin","Presence","Voice")]
	[string] $PolicyType = "ExternalAccess",

	# Specifies install option, which creates a scheduled task for the script. It does NOT create the required Exchange receive connector.
	[parameter(ValueFromPipelineByPropertyName, ParameterSetName = "Install")]
	[switch] $Install
)
[string] $ScriptVersion = "1.7"
[string] $ScriptName = $MyInvocation.MyCommand.Name
[string] $ScriptPathAndName = $MyInvocation.MyCommand.Definition

#region functions
function Get-ADNestedGroupMembers{
	[cmdletBinding(SupportsShouldProcess)]
	param (
		[parameter(ValueFromPipeline, ValueFromPipelineByPropertyName)]
		[String] $Group
	)
	Write-Verbose "Gathering group membership of $GroupDN"
	$Members = Get-ADGroupMember -Identity $Group
	# $Members | % {
	$Members | ForEach-Object {
		if ($_.ObjectClass -eq "group") {
			Write-Verbose "Getting nested group membership of $_.distinguishedName"
			Get-ADNestedGroupMembers -group $_.distinguishedName
		} else {
			$error.clear()
			[string] $name = $_.samaccountname
			Write-Verbose ""
			if ((Get-CsAdUser $name -ErrorAction SilentlyContinue).enabled){
				Write-Verbose "$name is enabled in Lync"
				[string] $cmd = "Grant-CS"+$PolicyType+"Policy -Identity "+$name+" -PolicyName `"$PolicyName`""
				Write-Verbose "Granting $PolicyType policy `"$PolicyName`""
				Invoke-Expression $cmd
				if (-Not ($error)){
					Write-Verbose "Successfully granted `"$PolicyName`" to $name"
				}else{
					Write-Verbose "Failed granting `"$PolicyName`" to $name because of $error"
				}
			}else{
				Write-Verbose "$name is NOT enabled in Lync\Skype for Business"
			}
			# return $_
		}
	}
} # end function Get-ADNestedGroupMembers

function Get-UpdateInfo {
<#
.SYNOPSIS
  Queries an online XML source for version information to determine if a new version of the script is available.

.DESCRIPTION
  Queries an online XML source for version information to determine if a new version of the script is available.

.NOTES
  Version               : 1.2 - See changelog at http://www.ehloworld.com/3168 for fixes & changes introduced with each version
  Wish list             : Better error trapping
  Rights Required       : N/A
  Sched Task Required   : No
  Lync/Skype4B Version  : N/A
  Author/Copyright      : © Pat Richard, Office Servers and Services (Skype for Business) MVP - All Rights Reserved
  Email/Blog/Twitter    : pat@innervation.com  https://ucunleashed.com  @patrichard
  Donations             : https://www.paypal.me/PatRichard
  Dedicated Post        : http://www.ehloworld.com/3168
  Disclaimer            : You running this script means you won't blame author(s) if this breaks your stuff. This script is
                          provided AS IS without warranty of any kind. Author(s) disclaim all implied warranties including,
                          without limitation, any implied warranties of merchantability or of fitness for a particular
                          purpose. The entire risk arising out of the use or performance of the sample scripts and
                          documentation remains with you. In no event shall author(s) be liable for any damages whatsoever
                          (including, without limitation, damages for loss of business profits, business interruption,
                          loss of business information, or other pecuniary loss) arising out of the use of or inability
                          to use the script or documentation. Neither this script, nor any part of it other than those
                          parts that are explicitly copied from others, may be republished without author(s) express written 
                          permission.
  Acknowledgements      : Reading XML files 
                          http://stackoverflow.com/questions/18509358/how-to-read-xml-in-powershell
                          http://stackoverflow.com/questions/20433932/determine-xml-node-exists
  Assumptions           : ExecutionPolicy of AllSigned (recommended), RemoteSigned, or Unrestricted (not recommended)
  Limitations           : 
  Known issues          : 

.LINK
  http://www.ehloworld.com/3168

.EXAMPLE
  Get-UpdateInfo -Article 123

  Description
  -----------
  Runs function to check for updates to article/script 123.

.INPUTS
  None. You cannot pipe objects to this script.
#>
  [CmdletBinding(SupportsShouldProcess, SupportsPaging)]
  param (
    [string] $article
  )
  [bool] $HasInternetAccess = ([Activator]::CreateInstance([Type]::GetTypeFromCLSID([Guid]'{DCB00C01-570F-4A9B-8D69-199FDBA5723B}')).IsConnectedToInternet)
  if ($HasInternetAccess){
    [xml] $xml = (New-Object System.Net.WebClient).DownloadString("http://www.ehloworld.com/downloads/version.xml")
    [string] $Ga = ($xml.catalog.article | Where-Object {$_.id -eq $article}).version
    if (($xml.catalog.article | Where-Object {$_.id -eq $article}).versionInfo){
      [string] $changelog = "This version includes: " + ($xml.catalog.article | Where-Object {$_.id -eq $article}).versionInfo
    }
    if ($Ga -gt $ScriptVersion){
      Write-Log -Level Warn -Message "Outdated version. Version $Ga is latest version. Prompting user to upgrade." -NoConsole
      if ((New-Popup -Message "Version $Ga is now available. $changelog `n`nWould you like to download it?" -Title "New version available!" -Buttons YesNo -Icon Question) -eq 6){
        Write-Log -Message "User elected to download latest version." -Indent 1 -NoConsole
        Write-Log -Message "Opening Internet Explorer and going to http://www.ehloworld.com/$article#downloads." -NoConsole -Indent 1
        Start-Process "http://www.ehloworld.com/$article#downloads"
        Write-Log -Level Warn -Message "Script is exiting. Please start the new version of the script once you've downloaded it." -NoLog
        Exit
      }else{
        Write-Log -Level Warn -Message "User elected NOT to download latest version." -Indent 1 -NoConsole
      }
    }elseif ($Ga -eq $ScriptVersion){
      Write-Log -Message "Current version. Version $Ga is latest version." -NoConsole
    }elseif (-Not ($Ga)) {
      # Write-Log -Level Warn -Message "Possibly regressed version. Version $Ga is latest version." -NoConsole
      Write-Log -Level Warn -Message "No version info available" -NoConsole
    }
  }else{
    Write-Log -Level Warn -Message "Unable to check online for update info." -NoConsole
  }
} # end function function Get-UpdateInfo

function Install {
	[cmdletBinding(SupportsShouldProcess)]
	param()
	BEGIN	{
		$error.clear()
		Write-Output "Creating scheduled task `"$ScriptName`"..."
		$TaskCreds = Get-Credential("$env:userdnsdomain\$env:username")
		$TaskPassword = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR($TaskCreds.Password))
	}
	PROCESS	{
		schtasks /create /tn $ScriptName /tr "$env:windir\system32\windowspowershell\v1.0\powershell.exe -noprofile -command $ScriptPathAndName" /sc Hourly /MO 4 /ru $TaskCreds.UserName /rp $TaskPassword | Out-Null
		if (-Not ($error)){
			Write-Host "Installation complete!" -ForegroundColor green
		}else{
			Write-Host "Installation failed!" -ForegroundColor red
			exit
		}
	}
	END	{
		Remove-Variable TaskCreds
		Remove-Variable TaskPassword
	}
} # end function Install

function Remove-ScriptVariables {
	[cmdletBinding(SupportsShouldProcess)]
	param(
		[parameter(ValueFromPipeline, ValueFromPipelineByPropertyName)]
		[string] $path
	)
	$result = Get-Content $path |
	ForEach {
		if ( $_ -match '(\$.*?)\s*=') {
			$matches[1]  | ? { $_ -notlike '*.*' -and $_ -notmatch 'result' -and $_ -notmatch 'env:'}
		}
	}
	ForEach ($v in ($result | Sort-Object | Get-Unique)){
		Remove-Variable ($v.replace("$","")) -ErrorAction SilentlyContinue
	}
} # end function Remove-ScriptVariables

function Set-ModuleStatus {
	[CmdletBinding(SupportsShouldProcess)]
	param	(
		[parameter(ValueFromPipeline, ValueFromPipelineByPropertyName, Mandatory, HelpMessage = "No module name specified!")]
		[ValidateNotNullOrEmpty()]
		[string] $name
	)
	PROCESS{
		# Executes once for each pipeline object
		# the $_ variable represents the current input object
		if (!(Get-Module -name "$name")) {
			if (Get-Module -ListAvailable | Where-Object Name -eq "$name") {
				Import-Module -Name "$name"
				# module was imported
				# return $true
			} else {
				# module was not available
				# return $false
			}
		} else {
			# Write-Output "$_ module already imported"
			# return $true
		}
	} # end PROCESS
} # end function Set-ModuleStatus
#endregion functions

Get-UpdateInfo -Article 1348

if ($install){
	Write-Verbose "Install mode"
	Install
	Exit
}

$error.clear()
Write-Verbose "Loading modules"

Set-ModuleStatus -Name Lync
Set-ModuleStatus -Name ActiveDirectory

Get-ADNestedGroupMembers -Group $GroupDN

Remove-ScriptVariables $ScriptPathAndName

# SIG # Begin signature block
# MIIcdgYJKoZIhvcNAQcCoIIcZzCCHGMCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQU6Orq9OMG6VePxDsiymoyijcf
# CzmgghelMIIFLjCCBBagAwIBAgIQA6cFj2F2VqaerCviGAu9jTANBgkqhkiG9w0B
# AQsFADByMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMTEwLwYDVQQDEyhEaWdpQ2VydCBTSEEyIEFz
# c3VyZWQgSUQgQ29kZSBTaWduaW5nIENBMB4XDTE2MTAxMjAwMDAwMFoXDTE3MTIy
# MTEyMDAwMFowazELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAk1JMRkwFwYDVQQHExBT
# dGVybGluZyBIZWlnaHRzMRkwFwYDVQQKExBJbm5lcnZhdGlvbiwgTExDMRkwFwYD
# VQQDExBJbm5lcnZhdGlvbiwgTExDMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIB
# CgKCAQEAvVNbtzSVPbpRQfYrhU0lPzCnmYKS2DFpgrL8gI8qUsyojdQJRjF8q5YQ
# uIi92Uwl/hxWcvCf0dU/EbzRZudOr8tsbkLA6x1R7O5Fsd6Q2fwMN4YmhFDKUcxf
# Gjy1byOdEhSQZqUc4xp6T+prcIi1j4EDSp5R0TQSRiKUK5HoqanhRyGfqJjkEOep
# KB9soxNVkB+jTXsIjX1AeuUJZS/m+1rzWf75laoid+rna7KQtJO+DkVNSa96tTH2
# 1Fs/C78UzWBLA5I2gQAgEA3alkMzB7rIr34Grt251O+yMSyCcvjb70AVcKohoS7n
# 2Zk522EK3jJw3F8euSWMbXNegJtPjwIDAQABo4IBxTCCAcEwHwYDVR0jBBgwFoAU
# WsS5eyoKo6XqcQPAYPkt9mV1DlgwHQYDVR0OBBYEFE+s6yTjN7ocNskZ2gxbJ3Ul
# xBo6MA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzB3BgNVHR8E
# cDBuMDWgM6Axhi9odHRwOi8vY3JsMy5kaWdpY2VydC5jb20vc2hhMi1hc3N1cmVk
# LWNzLWcxLmNybDA1oDOgMYYvaHR0cDovL2NybDQuZGlnaWNlcnQuY29tL3NoYTIt
# YXNzdXJlZC1jcy1nMS5jcmwwTAYDVR0gBEUwQzA3BglghkgBhv1sAwEwKjAoBggr
# BgEFBQcCARYcaHR0cHM6Ly93d3cuZGlnaWNlcnQuY29tL0NQUzAIBgZngQwBBAEw
# gYQGCCsGAQUFBwEBBHgwdjAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNl
# cnQuY29tME4GCCsGAQUFBzAChkJodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20v
# RGlnaUNlcnRTSEEyQXNzdXJlZElEQ29kZVNpZ25pbmdDQS5jcnQwDAYDVR0TAQH/
# BAIwADANBgkqhkiG9w0BAQsFAAOCAQEAL3G7W2y46ZvBae5m6LR+rZu1yA1kkmk5
# 55rTY9lA9xCd0jrhY3cPzwKxGFy82surJeWDWbDIao7wqGXIfDrDgFahdM63btdl
# LWJacdfrM7l+82Zx22R30P/K5sVmk1UXr9eyOPzrYMiemUVru3xMNSZ4Md7lHrfb
# CODsEGjs+zWo4eqXWcnnOCxMd1cpr4PvKPonUhMAhCyFDD5TDzZDQIbHBDIvfb8j
# rg/cvZ2PArf5QZpLp5CiSGnGpC+O9rZhYbsOmcJVFMWMM0pMEw46T9HXHxHMfDC1
# oYTp8Xyc/JxiQ+084oNoGng4esFZIb18cJFXDfR0f4ie0pJTMvanqjCCBTAwggQY
# oAMCAQICEAQJGBtf1btmdVNDtW+VUAgwDQYJKoZIhvcNAQELBQAwZTELMAkGA1UE
# BhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2lj
# ZXJ0LmNvbTEkMCIGA1UEAxMbRGlnaUNlcnQgQXNzdXJlZCBJRCBSb290IENBMB4X
# DTEzMTAyMjEyMDAwMFoXDTI4MTAyMjEyMDAwMFowcjELMAkGA1UEBhMCVVMxFTAT
# BgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEx
# MC8GA1UEAxMoRGlnaUNlcnQgU0hBMiBBc3N1cmVkIElEIENvZGUgU2lnbmluZyBD
# QTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAPjTsxx/DhGvZ3cH0wsx
# SRnP0PtFmbE620T1f+Wondsy13Hqdp0FLreP+pJDwKX5idQ3Gde2qvCchqXYJawO
# eSg6funRZ9PG+yknx9N7I5TkkSOWkHeC+aGEI2YSVDNQdLEoJrskacLCUvIUZ4qJ
# RdQtoaPpiCwgla4cSocI3wz14k1gGL6qxLKucDFmM3E+rHCiq85/6XzLkqHlOzEc
# z+ryCuRXu0q16XTmK/5sy350OTYNkO/ktU6kqepqCquE86xnTrXE94zRICUj6whk
# PlKWwfIPEvTFjg/BougsUfdzvL2FsWKDc0GCB+Q4i2pzINAPZHM8np+mM6n9Gd8l
# k9ECAwEAAaOCAc0wggHJMBIGA1UdEwEB/wQIMAYBAf8CAQAwDgYDVR0PAQH/BAQD
# AgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMDMHkGCCsGAQUFBwEBBG0wazAkBggrBgEF
# BQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tMEMGCCsGAQUFBzAChjdodHRw
# Oi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0Eu
# Y3J0MIGBBgNVHR8EejB4MDqgOKA2hjRodHRwOi8vY3JsNC5kaWdpY2VydC5jb20v
# RGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3JsMDqgOKA2hjRodHRwOi8vY3JsMy5k
# aWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3JsME8GA1UdIARI
# MEYwOAYKYIZIAYb9bAACBDAqMCgGCCsGAQUFBwIBFhxodHRwczovL3d3dy5kaWdp
# Y2VydC5jb20vQ1BTMAoGCGCGSAGG/WwDMB0GA1UdDgQWBBRaxLl7KgqjpepxA8Bg
# +S32ZXUOWDAfBgNVHSMEGDAWgBRF66Kv9JLLgjEtUYunpyGd823IDzANBgkqhkiG
# 9w0BAQsFAAOCAQEAPuwNWiSz8yLRFcgsfCUpdqgdXRwtOhrE7zBh134LYP3DPQ/E
# r4v97yrfIFU3sOH20ZJ1D1G0bqWOWuJeJIFOEKTuP3GOYw4TS63XX0R58zYUBor3
# nEZOXP+QsRsHDpEV+7qvtVHCjSSuJMbHJyqhKSgaOnEoAjwukaPAJRHinBRHoXpo
# aK+bp1wgXNlxsQyPu6j4xRJon89Ay0BEpRPw5mQMJQhCMrI2iiQC/i9yfhzXSUWW
# 6Fkd6fp0ZGuy62ZD2rOwjNXpDd32ASDOmTFjPQgaGLOBm0/GkxAG/AeB+ova+YJJ
# 92JuoVP6EpQYhS6SkepobEQysmah5xikmmRR7zCCBmowggVSoAMCAQICEAMBmgI6
# /1ixa9bV6uYX8GYwDQYJKoZIhvcNAQEFBQAwYjELMAkGA1UEBhMCVVMxFTATBgNV
# BAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEhMB8G
# A1UEAxMYRGlnaUNlcnQgQXNzdXJlZCBJRCBDQS0xMB4XDTE0MTAyMjAwMDAwMFoX
# DTI0MTAyMjAwMDAwMFowRzELMAkGA1UEBhMCVVMxETAPBgNVBAoTCERpZ2lDZXJ0
# MSUwIwYDVQQDExxEaWdpQ2VydCBUaW1lc3RhbXAgUmVzcG9uZGVyMIIBIjANBgkq
# hkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAo2Rd/Hyz4II14OD2xirmSXU7zG7gU6mf
# H2RZ5nxrf2uMnVX4kuOe1VpjWwJJUNmDzm9m7t3LhelfpfnUh3SIRDsZyeX1kZ/G
# FDmsJOqoSyyRicxeKPRktlC39RKzc5YKZ6O+YZ+u8/0SeHUOplsU/UUjjoZEVX0Y
# hgWMVYd5SEb3yg6Np95OX+Koti1ZAmGIYXIYaLm4fO7m5zQvMXeBMB+7NgGN7yfj
# 95rwTDFkjePr+hmHqH7P7IwMNlt6wXq4eMfJBi5GEMiN6ARg27xzdPpO2P6qQPGy
# znBGg+naQKFZOtkVCVeZVjCT88lhzNAIzGvsYkKRrALA76TwiRGPdwIDAQABo4ID
# NTCCAzEwDgYDVR0PAQH/BAQDAgeAMAwGA1UdEwEB/wQCMAAwFgYDVR0lAQH/BAww
# CgYIKwYBBQUHAwgwggG/BgNVHSAEggG2MIIBsjCCAaEGCWCGSAGG/WwHATCCAZIw
# KAYIKwYBBQUHAgEWHGh0dHBzOi8vd3d3LmRpZ2ljZXJ0LmNvbS9DUFMwggFkBggr
# BgEFBQcCAjCCAVYeggFSAEEAbgB5ACAAdQBzAGUAIABvAGYAIAB0AGgAaQBzACAA
# QwBlAHIAdABpAGYAaQBjAGEAdABlACAAYwBvAG4AcwB0AGkAdAB1AHQAZQBzACAA
# YQBjAGMAZQBwAHQAYQBuAGMAZQAgAG8AZgAgAHQAaABlACAARABpAGcAaQBDAGUA
# cgB0ACAAQwBQAC8AQwBQAFMAIABhAG4AZAAgAHQAaABlACAAUgBlAGwAeQBpAG4A
# ZwAgAFAAYQByAHQAeQAgAEEAZwByAGUAZQBtAGUAbgB0ACAAdwBoAGkAYwBoACAA
# bABpAG0AaQB0ACAAbABpAGEAYgBpAGwAaQB0AHkAIABhAG4AZAAgAGEAcgBlACAA
# aQBuAGMAbwByAHAAbwByAGEAdABlAGQAIABoAGUAcgBlAGkAbgAgAGIAeQAgAHIA
# ZQBmAGUAcgBlAG4AYwBlAC4wCwYJYIZIAYb9bAMVMB8GA1UdIwQYMBaAFBUAEisT
# mLKZB+0e36K+Vw0rZwLNMB0GA1UdDgQWBBRhWk0ktkkynUoqeRqDS/QeicHKfTB9
# BgNVHR8EdjB0MDigNqA0hjJodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNl
# cnRBc3N1cmVkSURDQS0xLmNybDA4oDagNIYyaHR0cDovL2NybDQuZGlnaWNlcnQu
# Y29tL0RpZ2lDZXJ0QXNzdXJlZElEQ0EtMS5jcmwwdwYIKwYBBQUHAQEEazBpMCQG
# CCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wQQYIKwYBBQUHMAKG
# NWh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRENB
# LTEuY3J0MA0GCSqGSIb3DQEBBQUAA4IBAQCdJX4bM02yJoFcm4bOIyAPgIfliP//
# sdRqLDHtOhcZcRfNqRu8WhY5AJ3jbITkWkD73gYBjDf6m7GdJH7+IKRXrVu3mrBg
# JuppVyFdNC8fcbCDlBkFazWQEKB7l8f2P+fiEUGmvWLZ8Cc9OB0obzpSCfDscGLT
# Ykuw4HOmksDTjjHYL+NtFxMG7uQDthSr849Dp3GdId0UyhVdkkHa+Q+B0Zl0DSbE
# Dn8btfWg8cZ3BigV6diT5VUW8LsKqxzbXEgnZsijiwoc5ZXarsQuWaBh3drzbaJh
# 6YoLbewSGL33VVRAA5Ira8JRwgpIr7DUbuD0FAo6G+OPPcqvao173NhEMIIGzTCC
# BbWgAwIBAgIQBv35A5YDreoACus/J7u6GzANBgkqhkiG9w0BAQUFADBlMQswCQYD
# VQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGln
# aWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVkIElEIFJvb3QgQ0Ew
# HhcNMDYxMTEwMDAwMDAwWhcNMjExMTEwMDAwMDAwWjBiMQswCQYDVQQGEwJVUzEV
# MBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29t
# MSEwHwYDVQQDExhEaWdpQ2VydCBBc3N1cmVkIElEIENBLTEwggEiMA0GCSqGSIb3
# DQEBAQUAA4IBDwAwggEKAoIBAQDogi2Z+crCQpWlgHNAcNKeVlRcqcTSQQaPyTP8
# TUWRXIGf7Syc+BZZ3561JBXCmLm0d0ncicQK2q/LXmvtrbBxMevPOkAMRk2T7It6
# NggDqww0/hhJgv7HxzFIgHweog+SDlDJxofrNj/YMMP/pvf7os1vcyP+rFYFkPAy
# IRaJxnCI+QWXfaPHQ90C6Ds97bFBo+0/vtuVSMTuHrPyvAwrmdDGXRJCgeGDboJz
# PyZLFJCuWWYKxI2+0s4Grq2Eb0iEm09AufFM8q+Y+/bOQF1c9qjxL6/siSLyaxhl
# scFzrdfx2M8eCnRcQrhofrfVdwonVnwPYqQ/MhRglf0HBKIJAgMBAAGjggN6MIID
# djAOBgNVHQ8BAf8EBAMCAYYwOwYDVR0lBDQwMgYIKwYBBQUHAwEGCCsGAQUFBwMC
# BggrBgEFBQcDAwYIKwYBBQUHAwQGCCsGAQUFBwMIMIIB0gYDVR0gBIIByTCCAcUw
# ggG0BgpghkgBhv1sAAEEMIIBpDA6BggrBgEFBQcCARYuaHR0cDovL3d3dy5kaWdp
# Y2VydC5jb20vc3NsLWNwcy1yZXBvc2l0b3J5Lmh0bTCCAWQGCCsGAQUFBwICMIIB
# Vh6CAVIAQQBuAHkAIAB1AHMAZQAgAG8AZgAgAHQAaABpAHMAIABDAGUAcgB0AGkA
# ZgBpAGMAYQB0AGUAIABjAG8AbgBzAHQAaQB0AHUAdABlAHMAIABhAGMAYwBlAHAA
# dABhAG4AYwBlACAAbwBmACAAdABoAGUAIABEAGkAZwBpAEMAZQByAHQAIABDAFAA
# LwBDAFAAUwAgAGEAbgBkACAAdABoAGUAIABSAGUAbAB5AGkAbgBnACAAUABhAHIA
# dAB5ACAAQQBnAHIAZQBlAG0AZQBuAHQAIAB3AGgAaQBjAGgAIABsAGkAbQBpAHQA
# IABsAGkAYQBiAGkAbABpAHQAeQAgAGEAbgBkACAAYQByAGUAIABpAG4AYwBvAHIA
# cABvAHIAYQB0AGUAZAAgAGgAZQByAGUAaQBuACAAYgB5ACAAcgBlAGYAZQByAGUA
# bgBjAGUALjALBglghkgBhv1sAxUwEgYDVR0TAQH/BAgwBgEB/wIBADB5BggrBgEF
# BQcBAQRtMGswJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBD
# BggrBgEFBQcwAoY3aHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0
# QXNzdXJlZElEUm9vdENBLmNydDCBgQYDVR0fBHoweDA6oDigNoY0aHR0cDovL2Ny
# bDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENBLmNybDA6oDig
# NoY0aHR0cDovL2NybDQuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9v
# dENBLmNybDAdBgNVHQ4EFgQUFQASKxOYspkH7R7for5XDStnAs0wHwYDVR0jBBgw
# FoAUReuir/SSy4IxLVGLp6chnfNtyA8wDQYJKoZIhvcNAQEFBQADggEBAEZQPsm3
# KCSnOB22WymvUs9S6TFHq1Zce9UNC0Gz7+x1H3Q48rJcYaKclcNQ5IK5I9G6OoZy
# rTh4rHVdFxc0ckeFlFbR67s2hHfMJKXzBBlVqefj56tizfuLLZDCwNK1lL1eT7EF
# 0g49GqkUW6aGMWKoqDPkmzmnxPXOHXh2lCVz5Cqrz5x2S+1fwksW5EtwTACJHvzF
# ebxMElf+X+EevAJdqP77BzhPDcZdkbkPZ0XN1oPt55INjbFpjE/7WeAjD9KqrgB8
# 7pxCDs+R1ye3Fu4Pw718CqDuLAhVhSK46xgaTfwqIa1JMYNHlXdx3LEbS0scEJx3
# FMGdTy9alQgpECYxggQ7MIIENwIBATCBhjByMQswCQYDVQQGEwJVUzEVMBMGA1UE
# ChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMTEwLwYD
# VQQDEyhEaWdpQ2VydCBTSEEyIEFzc3VyZWQgSUQgQ29kZSBTaWduaW5nIENBAhAD
# pwWPYXZWpp6sK+IYC72NMAkGBSsOAwIaBQCgeDAYBgorBgEEAYI3AgEMMQowCKAC
# gAChAoAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEEMBwGCisGAQQBgjcCAQsx
# DjAMBgorBgEEAYI3AgEVMCMGCSqGSIb3DQEJBDEWBBS9LOtNkeYuOD05MMivUawt
# TWBMMjANBgkqhkiG9w0BAQEFAASCAQAmZv2rFDhgmLVYawQstEmT7eFWFPW6Xuei
# cRcHdhv+XYxTnHD29jAGykb7ZsQmyRSsPppJH4TDcJWpeLQ5hfXojrsHyaBXO9p+
# H6dw+VGMDLFX9m0NYmX0VZ+aHcEwgNs7euu9yrmOGi9NR/5QvcOvm96wxCqaYulO
# mIbMGZg2b78bjJiOBZ0hbplyKuftaTmiAIdw1u0y+1d7ZkdEXvuJ8dY+KW/oIouF
# hHwnSP9X+wSRZZ5TtQvHlxlBE8eWIZ7eFQOqOj6zbLts/YnAfK8dlrzeUM0PeSIp
# jIi6+xB78lOggp9ayJ8tMVMOJEnpBjiWSQgSYcX8NTG+3D5G8GaLoYICDzCCAgsG
# CSqGSIb3DQEJBjGCAfwwggH4AgEBMHYwYjELMAkGA1UEBhMCVVMxFTATBgNVBAoT
# DERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEhMB8GA1UE
# AxMYRGlnaUNlcnQgQXNzdXJlZCBJRCBDQS0xAhADAZoCOv9YsWvW1ermF/BmMAkG
# BSsOAwIaBQCgXTAYBgkqhkiG9w0BCQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJ
# BTEPFw0xNzAyMDMwOTI2MzdaMCMGCSqGSIb3DQEJBDEWBBSN/J5zf5Rkq0F1Gfjr
# qZFmkPgvWjANBgkqhkiG9w0BAQEFAASCAQA810bURmRS2Z+KuiNQKhSXR9GGobxK
# ODAp5zauN0eKPhVsfaXBpTB3HHjPad1OEw3WS+bjG+ov7Z+hmP9xY7uz6RjddJct
# rwJ2X5+CHD0xiebakYXTUbVVF/flTL53I3iPw0DmH9dozDa/5FtCHJCox2ExZBzD
# WPYvAPGFp0JdUnhYOQeG1haJAaHa4bo7/Ag2Vug7m4+iAW6yWpGN9NMFiMg6GI56
# 7h/YRVbkazgdCXKd3jk69fDaeGul8XMOedotZrlfct4LdOE9ssi8ZDJJIYY9Nmav
# uZJ6ejVGmqiuk7ua4TtmdBIR11qFbtstzuUbgZAIGIfxus5adCiRmQYs
# SIG # End signature block
